@@ -4,7 +4,6 @@
 // Author:      Vadim Zeitlin
 // Modified by:
 // Created:     18.02.01
-// RCS-ID:      $Id: scrtimer.h,v 1.8 2006/06/08 11:25:17 ABX Exp $
 // Copyright:   (c) 2001 SciTech Software, Inc. (www.scitechsoft.com)
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -25,7 +24,7 @@
 // pressed
 // ----------------------------------------------------------------------------
 
-class WXDLLEXPORT wxScrollTimer : public wxTimer
+class WXDLLIMPEXP_CORE wxScrollTimer : public wxTimer
 {
 public:
     // default ctor
