@@ -4,7 +4,6 @@
 // Author:      Lukasz Michalski
 // Modified by:
 // Created:     27.06.2005
-// RCS-ID:      $Id$
 // Copyright:   (c) 2005 Lukasz Michalski <lmichalski@sf.net>
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -72,7 +71,7 @@ WX_DECLARE_LIST(ThreadWorker, TList);
 WX_DECLARE_LIST(EventWorker, EList);
 
 class Client : public wxApp {
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 public:
     void RemoveEventWorker(EventWorker* p_worker);
 private:
@@ -141,7 +140,7 @@ private:
 
 class EventWorker : public wxEvtHandler
 {
-    DECLARE_EVENT_TABLE()
+    wxDECLARE_EVENT_TABLE();
 public:
     EventWorker(const wxString& p_host, char* p_buf, int p_size);
     void Run();
@@ -306,9 +305,9 @@ Client::OnExit()
     return 0;
 }
 
-// Create buffer to be sent by client. Buffer contains test indicator 
+// Create buffer to be sent by client. Buffer contains test indicator
 // message size and place for data
-// msgsize parameter contains size of data in bytes and 
+// msgsize parameter contains size of data in bytes and
 // if input value does not fit into 250 bytes then
 // on exit is updated to new value that is multiply of 1024 bytes
 char*
@@ -318,7 +317,7 @@ Client::CreateBuffer(int* msgsize)
     char* buf;
     //if message should have more than 256 bytes then set it as
     //test3 for compatibility with GUI server sample
-    if ((*msgsize) > 250) 
+    if ((*msgsize) > 250)
     {
         //send at least one kb of data
         int size = (*msgsize)/1024 + 1;
@@ -505,10 +504,10 @@ Client::OnTimerEvent(wxTimerEvent&) {
     dumpStatistics();
 }
 
-BEGIN_EVENT_TABLE(Client,wxEvtHandler)
+wxBEGIN_EVENT_TABLE(Client,wxEvtHandler)
     EVT_WORKER(Client::OnWorkerEvent)
     EVT_TIMER(wxID_ANY,Client::OnTimerEvent)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 
 
@@ -647,9 +646,9 @@ EventWorker::~EventWorker() {
     delete [] m_inbuf;
 }
 
-BEGIN_EVENT_TABLE(EventWorker,wxEvtHandler)
+wxBEGIN_EVENT_TABLE(EventWorker,wxEvtHandler)
     EVT_SOCKET(wxID_ANY,EventWorker::OnSocketEvent)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 
 ThreadWorker::ThreadWorker(const wxString& p_host, char* p_buf, int p_size)
