@@ -4,7 +4,6 @@
 // Author:      Guillermo Rodriguez Garcia <guille@iies.es>
 // Modified by:
 // Created:     1999/09/19
-// RCS-ID:      $Id$
 // Copyright:   (c) 1999 Guillermo Rodriguez Garcia
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -32,6 +31,7 @@
 #include "wx/socket.h"
 #include "wx/url.h"
 #include "wx/sstream.h"
+#include "wx/thread.h"
 #include <memory>
 
 // --------------------------------------------------------------------------
@@ -39,7 +39,7 @@
 // --------------------------------------------------------------------------
 
 // the application icon
-#if !defined(__WXMSW__) && !defined(__WXPM__)
+#ifndef wxHAS_IMAGES_IN_RESOURCES
     #include "../sample.xpm"
 #endif
 
@@ -102,7 +102,7 @@ private:
   bool            m_busy;
 
   // any class wishing to process wxWidgets events must use this macro
-  DECLARE_EVENT_TABLE()
+  wxDECLARE_EVENT_TABLE();
 };
 
 // simple helper class to log start and end of each test
@@ -154,7 +154,7 @@ enum
 // event tables and other macros for wxWidgets
 // --------------------------------------------------------------------------
 
-BEGIN_EVENT_TABLE(MyFrame, wxFrame)
+wxBEGIN_EVENT_TABLE(MyFrame, wxFrame)
   EVT_MENU(CLIENT_QUIT,     MyFrame::OnQuit)
   EVT_MENU(CLIENT_ABOUT,    MyFrame::OnAbout)
   EVT_MENU(CLIENT_OPEN,     MyFrame::OnOpenConnection)
@@ -170,7 +170,7 @@ BEGIN_EVENT_TABLE(MyFrame, wxFrame)
   EVT_MENU(CLIENT_TESTURL,  MyFrame::OnTestURL)
 #endif
   EVT_SOCKET(SOCKET_ID,     MyFrame::OnSocketEvent)
-END_EVENT_TABLE()
+wxEND_EVENT_TABLE()
 
 IMPLEMENT_APP(MyApp)
 
@@ -190,9 +190,8 @@ bool MyApp::OnInit()
   // Create the main application window
   MyFrame *frame = new MyFrame();
 
-  // Show it and tell the application that it's our main window
+  // Show it
   frame->Show(true);
-  SetTopWindow(frame);
 
   // success
   return true;
@@ -212,7 +211,7 @@ MyFrame::MyFrame() : wxFrame((wxFrame *)NULL, wxID_ANY,
 
   // Make menus
   m_menuFile = new wxMenu();
-  m_menuFile->Append(CLIENT_ABOUT, _("&About...\tCtrl-A"), _("Show about dialog"));
+  m_menuFile->Append(CLIENT_ABOUT, _("&About\tCtrl-A"), _("Show about dialog"));
   m_menuFile->AppendSeparator();
   m_menuFile->Append(CLIENT_QUIT, _("E&xit\tAlt-X"), _("Quit client"));
 
@@ -577,23 +576,13 @@ void MyFrame::OnDatagram(wxCommandEvent& WXUNUSED(event))
 
 #if wxUSE_URL
 
-void MyFrame::OnTestURL(wxCommandEvent& WXUNUSED(event))
+void DoDownload(const wxString& urlname)
 {
-    // Ask for the URL
-    static wxString s_urlname("http://www.google.com/");
-    wxString urlname = wxGetTextFromUser
-                       (
-                        _("Enter an URL to get"),
-                        _("URL:"),
-                        s_urlname
-                       );
-    if ( urlname.empty() )
-        return; // cancelled by user
+    wxString testname("URL");
+    if ( !wxIsMainThread() )
+        testname += " in worker thread";
 
-    s_urlname = urlname;
-
-
-    TestLogger logtest("URL");
+    TestLogger logtest(testname);
 
     // Parse the URL
     wxURL url(urlname);
@@ -613,9 +602,9 @@ void MyFrame::OnTestURL(wxCommandEvent& WXUNUSED(event))
     }
 
     // Print the contents type and file size
-    wxLogMessage("Contents type: %s\nFile size: %i\nStarting to download...",
+    wxLogMessage("Contents type: %s\nFile size: %lu\nStarting to download...",
                  url.GetProtocol().GetContentType(),
-                 data->GetSize());
+                 static_cast<unsigned long>( data->GetSize() ));
 
     // Get the data
     wxStringOutputStream sout;
@@ -628,6 +617,51 @@ void MyFrame::OnTestURL(wxCommandEvent& WXUNUSED(event))
                  urlname, sout.GetString());
 }
 
+void MyFrame::OnTestURL(wxCommandEvent& WXUNUSED(event))
+{
+    // Ask for the URL
+    static wxString s_urlname("http://www.google.com/");
+    wxString urlname = wxGetTextFromUser
+                       (
+                        _("Enter an URL to get"),
+                        _("URL:"),
+                        s_urlname
+                       );
+    if ( urlname.empty() )
+        return; // cancelled by user
+
+    s_urlname = urlname;
+
+    // First do it in this thread.
+    DoDownload(urlname);
+
+    // And then also in a worker thread.
+#if wxUSE_THREADS
+    class DownloadThread : public wxThread
+    {
+    public:
+        explicit DownloadThread(const wxString& url): m_url(url)
+        {
+            Run();
+        }
+
+        virtual void* Entry()
+        {
+            DoDownload(m_url);
+
+            return NULL;
+        }
+
+    private:
+        const wxString m_url;
+    };
+
+    // NB: there is a race condition here, we don't check for this thread
+    // termination before exiting the application, don't do this in real code!
+    new DownloadThread(urlname);
+#endif // wxUSE_THREADS
+}
+
 #endif // wxUSE_URL
 
 void MyFrame::OnSocketEvent(wxSocketEvent& event)
