@@ -36,6 +36,7 @@
 #include "wx/dynload.h"
 #include "wx/scopeguard.h"
 #include "wx/filename.h"
+#include "wx/fontenc.h"
 
 #include "wx/confbase.h"        // for wxExpandEnvVars()
 
@@ -384,9 +385,9 @@ const wxChar* wxGetHomeDir(wxString *pstr)
         // Cygwin returns unix type path but that does not work well
         static wxChar windowsPath[MAX_PATH];
         #if CYGWIN_VERSION_DLL_MAJOR >= 1007
-            cygwin_conv_path(CCP_POSIX_TO_WIN_W, strDir, windowsPath, MAX_PATH);
+            cygwin_conv_path(CCP_POSIX_TO_WIN_W, strDir.c_str(), windowsPath, MAX_PATH);
         #else
-            cygwin_conv_to_full_win32_path(strDir, windowsPath);
+            cygwin_conv_to_full_win32_path(strDir.c_str(), windowsPath);
         #endif
         strDir = windowsPath;
     #endif
@@ -1333,6 +1334,12 @@ wxString wxGetOsDescription()
                             break;
                     }
                     break;
+
+                case 10:
+                    str = wxIsWindowsServer() == 1
+                            ? _("Windows Server 2016")
+                            : _("Windows 10");
+                    break;
             }
 
             if ( str.empty() )
@@ -1488,8 +1495,18 @@ wxWinVersion wxGetWinVersion()
 
                         case 2:
                             return wxWinVersion_8;
+
+                        case 3:
+                            // For now, map to wxWinVersion_8. In case program
+                            // does not have a manifest indicating 8.1 or 10
+                            // support, Windows already performs this mapping
+                            // for us.
+                            return wxWinVersion_8;
                     }
                     break;
+
+                case 10:
+                    return wxWinVersion_8;
             }
         default:
             // Do nothing just to silence GCC warning
@@ -1591,9 +1608,6 @@ extern WXDLLIMPEXP_BASE long wxEncodingToCharset(wxFontEncoding encoding)
 // looks up the vlaues in the registry and the new one which is more
 // politically correct and has more chances to work on other Windows versions
 // as well but the old version is still needed for !wxUSE_FONTMAP case
-#if wxUSE_FONTMAP
-
-#include "wx/fontmap.h"
 
 extern WXDLLIMPEXP_BASE long wxEncodingToCodepage(wxFontEncoding encoding)
 {
@@ -1658,7 +1672,7 @@ extern WXDLLIMPEXP_BASE long wxEncodingToCodepage(wxFontEncoding encoding)
         case wxFONTENCODING_MACCENTRALEUR:  ret = 10029; break;
         case wxFONTENCODING_MACCROATIAN:    ret = 10082; break;
         case wxFONTENCODING_MACICELANDIC:   ret = 10079; break;
-        case wxFONTENCODING_MACROMANIAN:    ret = 10009; break;
+        case wxFONTENCODING_MACROMANIAN:    ret = 10010; break;
 
         case wxFONTENCODING_ISO2022_JP:     ret = 50222; break;
 
@@ -1678,6 +1692,10 @@ extern WXDLLIMPEXP_BASE long wxEncodingToCodepage(wxFontEncoding encoding)
     return (long) ret;
 }
 
+#if wxUSE_FONTMAP
+
+#include "wx/fontmap.h"
+
 extern long wxCharsetToCodepage(const char *name)
 {
     // first get the font encoding for this charset
