@@ -4,7 +4,6 @@
 // Author:      David Webster
 // Modified by:
 // Created:     10/09/99
-// RCS-ID:      $Id: iniconf.cpp,v 1.13 2006/04/26 08:21:12 ABX Exp $
 // Copyright:   David Webster
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -255,11 +254,11 @@ bool wxIniConfig::IsEmpty() const
 
 //  GetPrivateProfileString(m_strGroup, NULL, "",
 //                          szBuf, WXSIZEOF(szBuf), m_strLocalFilename);
-    if ( !::IsEmpty(szBuf) )
+    if ( !::wxIsEmpty(szBuf) )
         return false;
 
 //  GetProfileString(m_strGroup, NULL, "", szBuf, WXSIZEOF(szBuf));
-//  if ( !::IsEmpty(szBuf) )
+//  if ( !::wxIsEmpty(szBuf) )
 //      return false;
 
     return true;
@@ -281,14 +280,14 @@ bool wxIniConfig::Read(const wxString& szKey, wxString *pstr) const
     // NB: the lpDefault param to GetPrivateProfileString can't be NULL
 //  GetPrivateProfileString(m_strGroup, strKey, "",
 //                          szBuf, WXSIZEOF(szBuf), m_strLocalFilename);
-    if ( ::IsEmpty((PSZ)szBuf) )
+    if ( ::wxIsEmpty((PSZ)szBuf) )
     {
         // now look in win.ini
         wxString strKey = GetKeyName(path.Name());
         // GetProfileString(m_strGroup, strKey, "", szBuf, WXSIZEOF(szBuf));
     }
 
-    if ( ::IsEmpty((PSZ)szBuf) )
+    if ( ::wxIsEmpty((PSZ)szBuf) )
     {
         return false;
     }
@@ -310,14 +309,14 @@ bool wxIniConfig::Read(const wxString& szKey, wxString *pstr,
     // NB: the lpDefault param to GetPrivateProfileString can't be NULL
     // GetPrivateProfileString(m_strGroup, strKey, "",
     //                         szBuf, WXSIZEOF(szBuf), m_strLocalFilename);
-    if ( ::IsEmpty((PSZ)szBuf) )
+    if ( ::wxIsEmpty((PSZ)szBuf) )
     {
       // now look in win.ini
       wxString strKey = GetKeyName(path.Name());
       // GetProfileString(m_strGroup, strKey, "", szBuf, WXSIZEOF(szBuf));
     }
 
-    if ( ::IsEmpty((PSZ)szBuf) )
+    if ( ::wxIsEmpty((PSZ)szBuf) )
     {
         *pstr = szDefault;
         return false;
@@ -372,7 +371,9 @@ bool wxIniConfig::Write(const wxString& szKey, const wxString& WXUNUSED(szValue)
 //                                       szValue, m_strLocalFilename) != 0;
 
     if ( !bOk )
+    {
         wxLogLastError(wxT("WritePrivateProfileString"));
+    }
 
     return bOk;
 }
@@ -417,7 +418,9 @@ bool wxIniConfig::DeleteEntry(const wxString& szKey, bool bGroupIfEmptyAlso)
                       //                             NULL, m_strLocalFilename) != 0;
 
     if ( !bOk )
+    {
         wxLogLastError(wxT("WritePrivateProfileString"));
+    }
 
     return bOk;
 }
@@ -432,7 +435,9 @@ bool wxIniConfig::DeleteGroup(const wxString& szKey)
                     //                             NULL, m_strLocalFilename) != 0;
 
   if ( !bOk )
+  {
     wxLogLastError(wxT("WritePrivateProfileString"));
+  }
 
   return bOk;
 }
