@@ -4,7 +4,6 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     17/09/98
-// RCS-ID:      $Id: pen.cpp 39724 2006-06-14 17:09:39Z ABX $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -24,16 +23,16 @@
 // wxPen
 //-----------------------------------------------------------------------------
 
-class wxPenRefData: public wxObjectRefData
+class wxPenRefData : public wxGDIRefData
 {
 public:
     wxPenRefData()
     {
         m_width = 1;
-        m_style = wxSOLID;
+        m_style = wxPENSTYLE_SOLID;
         m_joinStyle = wxJOIN_ROUND;
         m_capStyle = wxCAP_ROUND;
-        m_dash = (wxX11Dash*) NULL;
+        m_dash = NULL;
         m_countDashes = 0;
     }
 
@@ -63,9 +62,9 @@ public:
     }
 
     int        m_width;
-    int        m_style;
-    int        m_joinStyle;
-    int        m_capStyle;
+    wxPenStyle m_style;
+    wxPenJoin  m_joinStyle;
+    wxPenCap   m_capStyle;
     wxColour   m_colour;
     int        m_countDashes;
     wxBitmap   m_stipple;
@@ -78,7 +77,7 @@ public:
 
 IMPLEMENT_DYNAMIC_CLASS(wxPen,wxGDIObject)
 
-wxPen::wxPen( const wxColour &colour, int width, int style )
+wxPen::wxPen( const wxColour &colour, int width, wxPenStyle style )
 {
     m_refData = new wxPenRefData();
     M_PENDATA->m_width = width;
@@ -86,17 +85,27 @@ wxPen::wxPen( const wxColour &colour, int width, int style )
     M_PENDATA->m_colour = colour;
 }
 
+#if FUTURE_WXWIN_COMPATIBILITY_3_0
+wxPen::wxPen(const wxColour& colour, int width, int style)
+{
+    m_refData = new wxPenRefData();
+    M_PENDATA->m_width = width;
+    M_PENDATA->m_style = (wxPenStyle)style;
+    M_PENDATA->m_colour = colour;
+}
+#endif
+
 wxPen::~wxPen()
 {
     // m_refData unrefed in ~wxObject
 }
 
-wxObjectRefData *wxPen::CreateRefData() const
+wxGDIRefData *wxPen::CreateGDIRefData() const
 {
     return new wxPenRefData;
 }
 
-wxObjectRefData *wxPen::CloneRefData(const wxObjectRefData *data) const
+wxGDIRefData *wxPen::CloneGDIRefData(const wxGDIRefData *data) const
 {
     return new wxPenRefData(*(wxPenRefData *)data);
 }
@@ -132,28 +141,28 @@ void wxPen::SetColour( unsigned char red, unsigned char green, unsigned char blu
     M_PENDATA->m_colour.Set( red, green, blue );
 }
 
-void wxPen::SetCap( int capStyle )
+void wxPen::SetCap( wxPenCap capStyle )
 {
     AllocExclusive();
 
     M_PENDATA->m_capStyle = capStyle;
 }
 
-void wxPen::SetJoin( int joinStyle )
+void wxPen::SetJoin( wxPenJoin joinStyle )
 {
     AllocExclusive();
 
     M_PENDATA->m_joinStyle = joinStyle;
 }
 
-void wxPen::SetStipple( wxBitmap *stipple )
+void wxPen::SetStipple( const wxBitmap& stipple )
 {
     AllocExclusive();
 
-    M_PENDATA->m_stipple = *stipple;
+    M_PENDATA->m_stipple = stipple;
 }
 
-void wxPen::SetStyle( int style )
+void wxPen::SetStyle( wxPenStyle style )
 {
     AllocExclusive();
 
@@ -169,8 +178,10 @@ void wxPen::SetWidth( int width )
 
 int wxPen::GetDashes( wxDash **ptr ) const
 {
-     *ptr = (M_PENDATA ? (wxDash*)M_PENDATA->m_dash : (wxDash*) NULL);
-     return (M_PENDATA ? M_PENDATA->m_countDashes : 0);
+    wxCHECK_MSG( IsOk(), -1, wxT("invalid pen") );
+
+    *ptr = (wxDash*)M_PENDATA->m_dash;
+    return M_PENDATA->m_countDashes;
 }
 
 int wxPen::GetDashCount() const
@@ -183,44 +194,44 @@ wxDash* wxPen::GetDash() const
     return (wxDash*)M_PENDATA->m_dash;
 }
 
-int wxPen::GetCap() const
+wxPenCap wxPen::GetCap() const
 {
-    wxCHECK_MSG( Ok(), -1, wxT("invalid pen") );
+    wxCHECK_MSG( IsOk(), wxCAP_INVALID, wxT("invalid pen") );
 
     return M_PENDATA->m_capStyle;
 }
 
-int wxPen::GetJoin() const
+wxPenJoin wxPen::GetJoin() const
 {
-    wxCHECK_MSG( Ok(), -1, wxT("invalid pen") );
+    wxCHECK_MSG( IsOk(), wxJOIN_INVALID, wxT("invalid pen") );
 
     return M_PENDATA->m_joinStyle;
 }
 
-int wxPen::GetStyle() const
+wxPenStyle wxPen::GetStyle() const
 {
-    wxCHECK_MSG( Ok(), -1, wxT("invalid pen") );
+    wxCHECK_MSG( IsOk(), wxPENSTYLE_INVALID, wxT("invalid pen") );
 
     return M_PENDATA->m_style;
 }
 
 int wxPen::GetWidth() const
 {
-    wxCHECK_MSG( Ok(), -1, wxT("invalid pen") );
+    wxCHECK_MSG( IsOk(), -1, wxT("invalid pen") );
 
     return M_PENDATA->m_width;
 }
 
-wxColour &wxPen::GetColour() const
+wxColour wxPen::GetColour() const
 {
-    wxCHECK_MSG( Ok(), wxNullColour, wxT("invalid pen") );
+    wxCHECK_MSG( IsOk(), wxNullColour, wxT("invalid pen") );
 
     return M_PENDATA->m_colour;
 }
 
 wxBitmap *wxPen::GetStipple() const
 {
-    wxCHECK_MSG( Ok(), &wxNullBitmap, wxT("invalid pen") );
+    wxCHECK_MSG( IsOk(), &wxNullBitmap, wxT("invalid pen") );
 
     return &M_PENDATA->m_stipple;
 }
